@@ -10,6 +10,8 @@ import json
 import itsdangerous
 import logging
 
+import requests
+
 log = logging.getLogger(__name__)
 
 LOGINCOOKIE = "devpi-login"
@@ -133,13 +135,15 @@ class PyPIView:
             parts = path.split("/")
             if len(parts) >= 2:
                 user, index = parts[:2]
-                if not self.db.user_indexconfig_get(user, index):
+                ixconfig = self.db.user_indexconfig_get(user, index)
+                if not ixconfig:
                     abort(404, "index %s/%s does not exist" %(user, index))
                 api.update({
                     "index": "/%s/%s/" % (user, index),
-                    "pypisubmit": "/%s/%s/" % (user, index),
                     "simpleindex": "/%s/%s/+simple/" % (user, index),
                 })
+                if ixconfig["type"] == "stage":
+                    api["pypisubmit"] = "/%s/%s/" % (user, index)
         apireturn(200, type="apiconfig", result=api)
 
     #
@@ -252,15 +256,54 @@ class PyPIView:
             indexes["%s/%s" % (user, name)] = val
         apireturn(200, type="list:indexconfig", result=indexes)
 
+    @route("/<user>/<index>/", method="PUSH")
+    def pushrelease(self, user, index):
+        pushdata = getjson()
+        name = pushdata["name"]
+        version = pushdata["version"]
+        posturl = pushdata["posturl"]
+        username = pushdata["username"]
+        password = pushdata["password"]
+        pypiauth = (username, password)
+        stage = self.getstage(user, index)
+        metadata = stage.get_metadata(name, version)
+        assert metadata
+        entries = stage.getreleaselinks(name)
+        matches = []
+        results = []
+        for entry in entries:
+            n, v = urlutil.guess_pkgname_and_version(entry.basename)
+            if n == name and str(v) == version:
+                matches.append(entry)
+        metadata[":action"] = "submit"
+        r = requests.post(posturl, data=metadata, auth=pypiauth)
+        ok_codes = (200, 201)
+        results.append((r.status_code, "register", name, version))
+        if r.status_code in ok_codes:
+            for entry in matches:
+                metadata[":action"] = "file_upload"
+                metadata["filetype"] = "sdist"  # XXX
+                basename = entry.basename
+                openfile = entry.FILE.filepath.open("rb")
+                r = requests.post(posturl, data=metadata, auth=pypiauth,
+                      files={"content": (basename, openfile)})
+                results.append((r.status_code, "upload", entry.relpath))
+        if r.status_code in ok_codes:
+            apireturn(200, result=results, type="actionlog")
+        else:
+            apireturn(502, result=results, type="actionlog")
+
     @route("/<user>/<index>/", method="POST")
     def submit(self, user, index):
+        if user == "root" and index == "pypi":
+            abort(404, "cannot submit to pypi mirror")
         self.require_user(user)
         try:
             action = request.forms[":action"]
         except KeyError:
             abort(400, ":action field not found")
-        log.debug("received POST action %r" %(action))
         stage = self.getstage(user, index)
+        log.debug("received POST action %r" %(action))
         if action == "submit":
             return self._register_metadata(stage, request.forms)
         elif action in ("doc_upload", "file_upload"):
@@ -397,7 +440,13 @@ class PyPIView:
         latest_packages = html.ul()
         for name in stage.getprojectnames_perstage():
             for entry in stage.getreleaselinks(name):
-                name, ver = urlutil.DistURL(entry.relpath).pkgname_and_version
+                if entry.eggfragment:
+                    continue
+                if entry.url:
+                    path = entry.url
+                else:
+                    path = entry.relpath
+                name, ver = urlutil.DistURL(path).pkgname_and_version
                 latest_packages.append(html.li(
                     html.a("%s-%s info page" % (name, ver),
                            href="%s/%s/" % (name, ver)),
