@@ -2,6 +2,7 @@
 import pytest
 import py
 import webtest
+import requests, json
 from bs4 import BeautifulSoup
 from webtest.forms import Upload
 from devpi_server.views import PyPIView
@@ -60,6 +61,16 @@ def test_simple_project(pypiurls, httpget, testapp):
     assert len(links) == 1
     assert links[0].get("href").endswith(path)
 
+def test_simple_project_pypi_egg(pypiurls, httpget, testapp):
+    httpget.setextsimple("py",
+        """<a href="http://bb.org/download/py.zip#egg=py-dev" />""")
+    r = testapp.get("/root/pypi/+simple/py/")
+    assert r.status_code == 200
+    links = BeautifulSoup(r.text).findAll("a")
+    assert len(links) == 1
+    r = testapp.get("/root/pypi/")
+    assert r.status_code == 200
+
 def test_simple_list(pypiurls, httpget, testapp):
     httpget.setextsimple("hello1", text="<html/>")
     httpget.setextsimple("hello2", text="<html/>")
@@ -94,11 +105,22 @@ def test_pkgserv(pypiurls, httpget, testapp):
     assert r.body == "123"
 
 def test_apiconfig(httpget, testapp):
-    r = testapp.get("/user/name/-api")
+    r = testapp.get("/user/name/+api")
     assert r.status_code == 404
+    r = testapp.get("/root/dev/+api")
+    assert r.status_code == 200
+    assert r.json["result"]["pypisubmit"]
+    r = testapp.get("/root/pypi/+api")
+    assert r.status_code == 200
+    assert not "pypisubmit" in r.json["result"]
+
     #for name in "pushrelease simpleindex login pypisubmit resultlog".split():
     #    assert name in r.json
     #
+    #
+def test_root_pypi(httpget, testapp):
+    r = testapp.get("/root/pypi/")
+    assert r.status_code == 200
 
 def test_register_metadata_and_get_description(httpget, db, mapp, testapp):
     mapp.create_and_login_user("user")
@@ -115,7 +137,7 @@ def test_register_metadata_and_get_description(httpget, db, mapp, testapp):
     assert r.status_code == 200
     assert "1.0" in r.json["result"]
 
-def test_upload(httpget, db, mapp, testapp):
+def test_upload_and_push_ok(httpget, db, mapp, testapp, monkeypatch):
     mapp.create_and_login_user("user")
     mapp.create_index("name")
     mapp.upload_file_pypi(
@@ -125,6 +147,45 @@ def test_upload(httpget, db, mapp, testapp):
     a = getfirstlink(r.text)
     assert "pkg1-2.6.tgz" in a.get("href")
 
+    # get root index page
+    r = testapp.get("/user/name/")
+    assert r.status_code == 200
+
+    # push
+    req = dict(name="pkg1", version="2.6", posturl="whatever",
+               username="user", password="password")
+    rec = []
+    def recpost(url, data, auth, files=None):
+        rec.append((url, data, auth, files))
+        class r:
+            status_code = 200
+        return r
+    monkeypatch.setattr(requests, "post", recpost)
+    body = json.dumps(req)
+    r = testapp.request("/user/name/", method="push", body=body,
+                        expect_errors=True)
+    assert r.status_code == 200
+    assert len(rec) == 2
+    assert rec[0][0] == "whatever"
+    assert rec[1][0] == "whatever"
+
+    # push with error
+    def posterror(url, data, auth, files=None):
+        class r:
+            status_code = 500
+        return r
+    monkeypatch.setattr(requests, "post", posterror)
+    r = testapp.request("/user/name/", method="push", body=body,
+                        expect_errors=True)
+    assert r.status_code == 502
+    result = r.json["result"]
+    assert len(result) == 1
+    assert result[0][0] == 500
+
+def test_upload_pypi_fails(httpget, db, mapp, testapp):
+    mapp.upload_file_pypi(
+            "root", "pypi", "pkg1-2.6.tgz", "123", "pkg1", "2.6", code=404)
+
 def test_upload_docs_too_large(httpget, db, mapp, testapp):
     from devpi_server.views import MAXDOCZIPSIZE
     mapp.create_and_login_user("user")
@@ -266,11 +327,12 @@ class Mapp:
         if code == 201:
             assert "created" in r.json["message"]
 
-    def upload_file_pypi(self, user, index, basename, content, name, version):
+    def upload_file_pypi(self, user, index, basename, content,
+                         name, version, code=200):
         r = self.testapp.post("/%s/%s/" % (user, index),
             {":action": "file_upload", "name": name, "version": version,
-             "content": Upload(basename, content)})
-        assert r.status_code == 200
+             "content": Upload(basename, content)}, expect_errors=True)
+        assert r.status_code == code
 
     def upload_doc(self, user, index, basename, content, name, version,
                          code=200):
